/* styles for the ecomment component
 *
 *  :organization: Logilab
 *  :copyright: 2003-2008 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
 *  :contact: http://www.logilab.fr/ -- mailto:contact@logilab.fr
 */

.commentBody {
  padding : 1px 1em 1px 12px;
  overflow: auto;
}

.commentInfo {
  padding : 1px 1em 1px 27px;
  margin-top : 0.8em;
  font-weight: bold;
  background: url("comment.png") top left no-repeat;
}

.replyto {
 font-weight: normal
}

li.comment {
  margin: 0px;
  padding: 0px;
  background-image: none;
}

ul.comment {
  margin: 0px;
  padding: 0px;
}

ul.comment ul {
  margin-left: 14px;
}

li.comment li.comment {
  margin-left: 2.8em;
}

.replyComment {
  padding-left: 30px;
}
